# -*- coding: utf-8 -*-

#1a) La conformazione di una proteina può essere descritta attraverso la 
# successione degli angoli diedri (phi, psi) tra le sue unità peptidiche.
# Si scriva una funzione pos_in_chain(pdbfile, phi0, deltaphi, psi0, deltapsi)
# che, in riferimento a un file PDB CHE CONTIENE UNA SOLA CATENA, restituisca 
# una lista di booleani in cui ciascun elemento indichi se, nella 
# corrispondente posizione della lista degli angoli (phi, psi), i valori 
# ricadono o no nell'intervallo specificato (valori espressi in radianti).
# Si suppone che i valori degli angoli siano compresi tra -math.pi e +mat.pi, 
# e che le ampiezze d'intervallo deltaphi/psi siano tra 0 e 2*math.pi 
# Tale funzione dovrà lanciare un'eccezione del tipo ValueError se il file PDB 
# contiene più di una catena.
# Suggerimento: trattare le catene proteiche caricate da file PDB come 
# istanze della classe "Polypeptide" di Biopython; a riguardo, leggere le note
# allegate.


from Bio.PDB.PDBParser import PDBParser
from Bio.PDB.Polypeptide import PPBuilder
import math 

PI2 = 2*math.pi

def pos_in_chain(pdbfile, phi0, deltaphi, psi0, deltapsi):
    
    def inrange(phi,psi):
        if phi and phi+PI2 <= phi0+deltaphi: phi+=PI2 #wraparound effect
        if psi and psi+PI2 <= psi0+deltapsi: psi+=PI2 #wraparound effect
        return (phi0 <= phi <= (phi0+deltaphi)) and \
               (psi0 <= psi <= (psi0+deltapsi))
               
    par = PDBParser()
    struct = par.get_structure('CONF1', pdbfile)
    nchains = list(struct.get_chains())
    if nchains:
        if len(nchains)!=1:
            raise ValueError  
    else:
        raise ValueError
        
    pp  = PPBuilder().build_peptides(struct) #to build a polypeptide
    l   = pp[0].get_phi_psi_list()           #to get the angles list
    
    return map(lambda x: inrange(*x),l)
    