# -*- coding: utf-8 -*-

#2a) Gli angoli diedri all'interno di un'alfa-elica vanno a collocarsi in 
# una zona del Ramachandran plot che possiamo localizzare approssimativamente 
# attraverso le seguenti relazioni (angoli espressi in gradi):
# -135 <= phi <= -30 ,  -60 <= psi <= +45 ,  -130 <= phi+psi <= -75 .
# Considerando che un'alfa-elica ha un numero minimo di residui pari a 4, si
# scriva una funzione guess_alpha(pdbfile) che, relativamente alla struttura 
# presente nel file PDB in input, stimi la presenza di alfa eliche sulla base 
# dell'ispezione della sequenza di angoli (phi, psi) della struttura.
# Si assuma che il file contenga un solo modello e una sola catena.
# La funzione dovrà restituire una lista in cui ciascun elemento dovrà essere 
# una tupla che individua inizio e lunghezza di un'alfa-elica; le tuple devono 
# essere in ordine crescente.
# Attenzione: per gli indici dei residui, si adotti la convenzione Python 
# (primo residuo con indice 0)


from Bio.PDB.PDBParser import PDBParser
from Bio.PDB.Polypeptide import PPBuilder
import math 

PI2 = 2*math.pi

phi0, phi1 = (math.pi/180)*(-135), (math.pi/180)*(-30)
psi0, psi1 = (math.pi/180)*(-60),  (math.pi/180)*(+45)
dia0, dia1 = (math.pi/180)*(-130), (math.pi/180)*(-75)


def guess_alpha(pdbfile):
    
    def inrange(phi,psi):
        return (phi0 <= phi <= phi1) and \
               (psi0 <= psi <= psi1) and \
               (dia0 <= phi+psi <=dia1)
               
    par = PDBParser()
    struct = par.get_structure('CONF1', pdbfile)
    nchains = list(struct.get_chains())
    if nchains:
        if len(nchains)!=1:
            raise ValueError  
    else:
        raise ValueError
        
    ppb = PPBuilder() #to build a polypeptide
    pp  = ppb.build_peptides(struct)
    l_angles  = pp[0].get_phi_psi_list()

    l_inrange = [False] + [inrange(*d) for d in l_angles[1:-2]] + [False]

    res = []
    cur_start, cur_len = None, 0

    for i, d in enumerate(l_inrange):
        if not d:
            if cur_len:
                if cur_len >= 4: res.append((cur_start,cur_len))
            cur_start, cur_len = None, 0
        elif not cur_len: 
            cur_start, cur_len = i, 1
        else:
            cur_len += 1
        
    return res

